from aiohttp import ClientSession

from .types import GoogleTranslateResult

import requests, trengine


class GoogleTranslator:
    @staticmethod
    def translate(
        text: str, to_language: str = "en", source_language: str = "auto"
    ) -> "GoogleTranslateResult":
        """Translate a text using google engine.

        Args:
            text (str): the text to translate.
            to_language (str, optional): The lang code of target lang. Defaults to "en".
            source_language (str, optional): Source lang of the text. Defaults to "auto".
        """
        response = requests.get(
            f"https://clients5.google.com/translate_a/t?client=at&sl={source_language}&tl={to_language}&q={text}",
            headers=trengine.HEADERS,
        )
        try:
            result = response.json()
        except Exception as e:
            raise BaseException(str(e))

        return GoogleTranslateResult.parse(result[0], to_language)


class AsyncGoogleTranslator:
    @staticmethod
    async def translate(
        text: str, to_language: str = "en", source_language: str = "auto"
    ) -> "GoogleTranslateResult":
        """Translate a text using google engine.

        Args:
            text (str): the text to translate.
            to_language (str, optional): The lang code of target lang. Defaults to "en".
            source_language (str, optional): Source lang of the text. Defaults to "auto".
        """
        async with ClientSession(headers=trengine.HEADERS) as session:
            async with session.get(
                f"https://clients5.google.com/translate_a/t?client=at&sl={source_language}&tl={to_language}&q={text}"
            ) as response:
                try:
                    result = await response.json()
                except Exception as e:
                    raise BaseException(str(e))

                return GoogleTranslateResult.parse(
                    result[0],
                    to_language,
                    source_language if source_language != "auto" else None,
                )



import os
import requests

from aiohttp import ClientSession, FormData

from .types import SpeechToTextResult
from .exceptions import ApiException

from json import dumps


class SpeechToText:
    @staticmethod
    def to_text(file_path: str, language: str = "en") -> "SpeechToTextResult":
        """Speech to text using google cloud.

        Args:
            file_path (str): The path to the audio file to be transcribed into text. This should be a valid path to an audio file
            language (str, optional): the source language code of the audio file. Defaults to "en". Supported languages are: en, ar
        """
        with open(file_path, "rb") as f:
            b = f.read()
        response = requests.post(
            "https://api.devrio.org/api/v1/SpeechToText/",
            data={
                "language": language,
            },
            files={"file": b},
        )
        try:
            result = response.json()
        except Exception as e:
            raise BaseException(str(e))

        if not result["ok"]:
            raise ApiException(dumps(result["error"], indent=2, ensure_ascii=False))

        return SpeechToTextResult.parse(result, file_path)


class AsyncSpeechToText:
    @staticmethod
    async def to_text(file_path: str, language: str = "en") -> "SpeechToTextResult":
        """Speech to text using google cloud.

        Args:
            file_path (str): The path to the audio file to be transcribed into text. This should be a valid path to an audio file
            language (str, optional): the source language code of the audio file. Defaults to "en". Supported languages are: en, ar
        """
        with open(file_path, "rb") as f:
            b = f.read()
        async with ClientSession() as session:
            form = FormData()
            form.add_field("file", b, filename=os.path.basename(file_path))
            form.add_field("language", language)
            async with session.request(
                "post", "https://api.devrio.org/api/v1/SpeechToText/", data=form
            ) as response:
                try:
                    result = await response.json()
                except Exception as e:
                    raise BaseException(str(e))

                if not result["ok"]:
                    raise ApiException(
                        dumps(result["error"], indent=2, ensure_ascii=False)
                    )

                return SpeechToTextResult.parse(result, file_path)



from aiohttp import ClientSession

from .types import AjaxTranslateResult
from .exceptions import ApiException

import requests, trengine


class Translator:
    @staticmethod
    def translate(
        text: str, translated_lang: str = "en", source_lang: str = None
    ) -> "AjaxTranslateResult":
        """Translate a text using tr engine.

        Args:
            text (str): the text to translate.
            translated_lang (str, optional): The lang code of target lang. Defaults to "en".
            source_lang (str, optional): Source lang of the text. Defaults to None.
        """
        source_lang = source_lang or Translator.detect(text)
        response = requests.post(
            "https://www.translate.com/translator/ajax_translate",
            data={
                "text_to_translate": text,
                "translated_lang": translated_lang,
                "source_lang": source_lang,
                "use_cache_only": "false",
            },
            headers=trengine.HEADERS,
        )
        try:
            result = response.json()
        except Exception as e:
            return BaseException(str(e))

        if not result["result"] == "success":
            raise ApiException(result["message"])

        return AjaxTranslateResult.parse(result, source_lang)

    @staticmethod
    def detect(text: str) -> str:
        """Detect language code from text.

        Args:
            text (str): The text.
        """
        response = requests.post(
            "https://www.translate.com/translator/ajax_lang_auto_detect",
            data={"text_to_translate": text},
            headers=trengine.HEADERS,
        )
        try:
            result = response.json()
        except Exception as e:
            return BaseException(str(e))
        if not result["result"] == "success":
            raise ApiException(result["message"])

        return result["language"]


class AsyncTranslator:
    @staticmethod
    async def translate(
        text: str, translated_lang: str = "en", source_lang: str = None
    ) -> "AjaxTranslateResult":
        """Translate a text using tr engine.

        Args:
            text (str): the text to translate.
            translated_lang (str, optional): The lang code of target lang. Defaults to "en".
            source_lang (str, optional): Source lang of the text. Defaults to None.
        """
        source_lang = source_lang or await AsyncTranslator.detect(text)
        async with ClientSession(headers=trengine.HEADERS) as session:
            async with session.post(
                "https://www.translate.com/translator/ajax_translate",
                data={
                    "text_to_translate": text,
                    "translated_lang": translated_lang,
                    "source_lang": source_lang,
                    "use_cache_only": "false",
                },
            ) as response:
                try:
                    result = await response.json()
                    print(result)
                except Exception as e:
                    return BaseException(str(e))
                if not result["result"] == "success":
                    raise ApiException(result["message"])

            return AjaxTranslateResult.parse(result, source_lang)

    @staticmethod
    async def detect(text: str) -> str:
        """Detect language code from text.

        Args:
            text (str): The text.
        """
        async with ClientSession() as session:
            async with session.post(
                "https://www.translate.com/translator/ajax_lang_auto_detect",
                data={"text_to_translate": text},
                headers=trengine.HEADERS,
            ) as response:
                try:
                    result = await response.json()
                except Exception as e:
                    return BaseException(str(e))
                if not result["result"] == "success":
                    raise ApiException(result["message"])

                return result["language"]



from aiohttp import ClientSession

from .types import HozoryTranslateResult
from .exceptions import ApiException

from json import dumps

import requests, trengine


class HozoryTranslator:
    @staticmethod
    def translate(text: str, target: str = "en") -> "HozoryTranslateResult":
        """Translate a text using hozory engine.

        Args:
            text (str): the text to translate.
            target (str, optional): The lang code of target lang. Defaults to "en".
        """
        response = requests.get(
            f"https://hozory.com/translate/?target={target}&text={text}",
            headers=trengine.HEADERS,
        )
        try:
            result = response.json()
        except Exception as e:
            raise BaseException(str(e))

        if not result["status"] == "ok":
            raise ApiException(dumps(result["result"], indent=2, ensure_ascii=False))

        return HozoryTranslateResult.parse(result["result"], target)


class AsyncHozoryTranslator:
    @staticmethod
    async def translate(text: str, target: str = "en") -> "HozoryTranslateResult":
        """Translate a text using hozory engine.

        Args:
            text (str): the text to translate.
            target (str, optional): The lang code of target lang. Defaults to "en".
        """
        async with ClientSession() as session:
            async with session.get(
                f"https://hozory.com/translate/?target={target}&text={text}",
                headers=trengine.HEADERS,
            ) as response:
                try:
                    result = await response.json()
                except Exception as e:
                    raise BaseException(str(e))

                if not result["status"] == "ok":
                    raise ApiException(
                        dumps(result["result"], indent=2, ensure_ascii=False)
                    )

                return HozoryTranslateResult.parse(result["result"], target)



import os
import requests

from aiohttp import ClientSession, FormData

from .types import IdentifyMusicResult
from .exceptions import ApiException

from json import dumps


class IdentifyMusic:
    @staticmethod
    def identify(file_path: str) -> "IdentifyMusicResult":
        """Identify music using shazam.

        Args:
            file_path (str): The path to the audio file to be identified. This should be a valid path to an audio file
        """
        with open(file_path, "rb") as f:
            b = f.read()
        response = requests.post(
            "https://api.devrio.org/api/v1/idetifymusic/",
            files={"file": (os.path.basename(file_path), b)},
        )
        try:
            result = response.json()
        except Exception as e:
            raise BaseException(str(e))

        if not result["ok"]:
            raise ApiException(dumps(result["error"], indent=2, ensure_ascii=False))

        return IdentifyMusicResult.parse(result, file_path)


class AsyncIdentifyMusic:
    @staticmethod
    async def identify(file_path: str) -> "IdentifyMusicResult":
        """Identify music using shazam.

        Args:
            file_path (str): The path to the audio file to be identified. This should be a valid path to an audio file
        """
        with open(file_path, "rb") as f:
            b = f.read()
        async with ClientSession() as session:
            form = FormData()
            form.add_field("file", b, filename=os.path.basename(file_path))
            async with session.request(
                "post", "https://api.devrio.org/api/v1/idetifymusic/", data=form
            ) as response:
                try:
                    result = await response.json()
                except Exception as e:
                    raise BaseException(str(e))
                if not result["ok"]:
                    raise ApiException(
                        dumps(result["error"], indent=2, ensure_ascii=False)
                    )

                return IdentifyMusicResult.parse(result, file_path)



from . import (
    google,
    hozory,
    speech_to_text,
    tdict,
    ocr,
    text_to_speech,
    identify_music,
    tr,
)

__all__ = ["AsyncEngine", "Engine"]

HEADERS = {
    "User-Agent": "GoogleTranslate/6.6.1.RC09.302039986 (Linux; U; Android 9; Redmi Note 8)"
}


class Engine:
    @property
    def tr(self) -> "tr.Translator":
        """Use tr engine.

        Returns:
            ajax.AjaxTranslator: Ajax Engine Class.
        """
        return tr.Translator

    @property
    def google(self) -> "google.GoogleTranslator":
        """Use google engine.

        Returns:
            google.GoogleTranslator: Google Engine Class.
        """
        return google.GoogleTranslator

    @property
    def hozory(self) -> "hozory.HozoryTranslator":
        """Use hozory engine.

        Returns:
            hozory.HozoryTranslator: Hozory Engine Class.
        """
        return hozory.HozoryTranslator

    @property
    def tdict(self) -> "tdict.TdictTranslator":
        """Use tdict engine.

        Returns:
            tdict.TdictTranslator: tdict Engine Class.
        """
        return tdict.TdictTranslator

    @property
    def ocr(self) -> "ocr.OCR":
        """Use OCR service.

        Returns:
            ocr.OCR: Ocr Class.
        """
        return ocr.OCR

    @property
    def speech_to_text(self) -> "speech_to_text.SpeechToText":
        """Use speech_to_text service.

        Returns:
            speech_to_text.SpeechToText: speech_to_text Engine Class.
        """
        return speech_to_text.SpeechToText

    @property
    def text_to_speech(self) -> "text_to_speech.TextToSpeech":
        """Use text_to_speech service.

        Returns:
            text_to_speech.TextToSpeech: text_to_speech Engine Class.
        """
        return text_to_speech.TextToSpeech

    @property
    def identify_music(self) -> "identify_music.IdentifyMusic":
        """Use identify_music service.

        Returns:
            identify_music.IdentifyMusic: identify_music Engine Class.
        """
        return identify_music.IdentifyMusic


class AsyncEngine:
    @property
    def tr(self) -> "tr.AsyncTranslator":
        """Use tr engine.

        Returns:
            ajax.AsyncAjaxTranslator: Ajax Engine Class.
        """
        return tr.AsyncTranslator

    @property
    def google(self) -> "google.AsyncGoogleTranslator":
        """Use google engine.

        Returns:
            google.AsyncGoogleTranslator: Google Engine Class.
        """
        return google.AsyncGoogleTranslator

    @property
    def hozory(self) -> "hozory.AsyncHozoryTranslator":
        """Use hozory engine.

        Returns:
            hozory.AsyncHozoryTranslator: Hozory Engine Class.
        """
        return hozory.AsyncHozoryTranslator

    @property
    def tdict(self) -> "tdict.AsyncTdictTranslator":
        """Use tdict engine.

        Returns:
            tdict.AsyncTdictTranslator: tdict Engine Class.
        """
        return tdict.AsyncTdictTranslator

    @property
    def ocr(self) -> "ocr.AsyncOCR":
        """Use OCR service.

        Returns:
            ocr.AsyncOCR: Ocr Class.
        """
        return ocr.AsyncOCR

    @property
    def speech_to_text(self) -> "speech_to_text.AsyncSpeechToText":
        """Use speech_to_text service.

        Returns:
            speech_to_text.AsyncSpeechToText: speech_to_text Engine Class.
        """
        return speech_to_text.AsyncSpeechToText

    @property
    def text_to_speech(self) -> "text_to_speech.AsyncTextToSpeech":
        """Use text_to_speech service.

        Returns:
            text_to_speech.AsyncTextToSpeech: text_to_speech Engine Class.
        """
        return text_to_speech.AsyncTextToSpeech

    @property
    def identify_music(self) -> "identify_music.AsyncIdentifyMusic":
        """Use identify_music service.

        Returns:
            identify_music.AsyncIdentifyMusic: identify_music Engine Class.
        """
        return identify_music.AsyncIdentifyMusic



from dataclasses import dataclass
from json import dumps


class Base:
    @staticmethod
    def default(obj: "Base"):
        return {
            attr: (getattr(obj, attr))
            for attr in filter(lambda x: not x.startswith("_"), obj.__dict__)
        }

    def __str__(self) -> str:
        return dumps(self, indent=4, default=Base.default, ensure_ascii=False)


@dataclass
class GoogleTranslateResult(Base):
    translated_text: str
    original_language: str
    dest_language: str

    @staticmethod
    def parse(d: list, dest: str, source: str = None) -> "GoogleTranslateResult":
        return GoogleTranslateResult(
            translated_text=" ".join([i for i in d[:-1]]) if isinstance(d, list) else d,
            original_language=d[-1] if source is None else source,
            dest_language=dest,
        )


@dataclass
class AjaxTranslateResult(Base):
    original_text: str
    translated_text: str
    translation_language: str
    original_language: str

    @staticmethod
    def parse(d: dict, org: str) -> "AjaxTranslateResult":
        return AjaxTranslateResult(
            original_text=d["original_text"],
            translated_text=d["translated_text"],
            translation_language=d["human_translation_details"]["translation_language"]
            if d["human_translation_details"]
            else None,
            original_language=org,
        )


@dataclass
class HozoryTranslateResult(Base):
    translated_text: str
    translation_language: str
    voice_link: str

    @staticmethod
    def parse(d: dict, dest: str) -> "HozoryTranslateResult":
        return HozoryTranslateResult(
            translated_text=d["translate"],
            translation_language=dest,
            voice_link=d["Voice_link"],
        )


@dataclass
class SpeechToTextResult(Base):
    status: bool
    file_path: str
    speech_text: str
    language: str

    @staticmethod
    def parse(d: dict, p: str) -> "SpeechToTextResult":
        return SpeechToTextResult(
            status=d["ok"],
            file_path=p,
            speech_text=d["result"],
            language=d["language"],
        )


@dataclass
class IdentifyMusicResult(Base):
    status: bool
    file_path: str
    tracks: list

    @staticmethod
    def parse(d: dict, p: str) -> "IdentifyMusicResult":
        return IdentifyMusicResult(
            status=d["ok"],
            file_path=p,
            tracks=d["data"],
        )


@dataclass
class TextToSpeechResult(Base):
    status: bool
    text: str
    language: str
    output_path: str

    @staticmethod
    def parse(d: dict, p: str) -> "TextToSpeechResult":
        return TextToSpeechResult(
            status=d["ok"],
            text=d["text"],
            output_path=p,
            language=d["language"],
        )



import requests
import base64
from aiohttp import ClientSession

from .types import TextToSpeechResult
from .exceptions import ApiException

from json import dumps


class TextToSpeech:
    @staticmethod
    def to_speech(
        text: str, output_path: str, language: str = "en"
    ) -> "TextToSpeechResult":
        """Text to speech using google translate.

        Args:
            text (str): The text to be converted into speech
            language (str, optional): the source language code of the text. Defaults to "en".
            out_put_path (str): The path to save the output audio file
        """
        response = requests.post(
            "https://api.devrio.org/api/v1/tts/",
            json={
                "language": language,
                "text": text,
            },
        )
        try:
            result = response.json()

            if not result["ok"]:
                raise ApiException(dumps(result["error"], indent=2, ensure_ascii=False))

            with open(output_path, "wb") as f:
                f.write(base64.b64decode(result["result"]))

        except Exception as e:
            raise BaseException(str(e))

        return TextToSpeechResult.parse(result, output_path)


class AsyncTextToSpeech:
    @staticmethod
    async def to_speech(
        text: str, output_path: str, language: str = "en"
    ) -> "TextToSpeechResult":
        """Text to speech using google translate.

        Args:
            text (str): The text to be converted into speech
            language (str, optional): the source language code of the text. Defaults to "en".
            out_put_path (str): The path to save the output audio file
        """
        async with ClientSession() as session:
            async with session.post(
                "https://api.devrio.org/api/v1/tts/",
                json={
                    "language": language,
                    "text": text,
                },
            ) as response:
                try:
                    result = await response.json()

                    if not result["ok"]:
                        raise ApiException(
                            dumps(result["error"], indent=2, ensure_ascii=False)
                        )

                    with open(output_path, "wb") as f:
                        f.write(base64.b64decode(result["result"]))

                except Exception as e:
                    raise BaseException(str(e))

        return TextToSpeechResult.parse(result, output_path)
